/*
Copyright Caroline et Serge COUDÉ, (02/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/

package net.sergecoude.TbiGeneral;

import java.util.*;
import net.n3.nanoxml.*;

public class Enregistrement  {

  private final static long serialVersionUID = 1L;
  
  public static int DATE_ASC = 0;
  public static int DATE_DESC = 1;
  public static int RANG_ASC = 2;
  public static int RANG_DESC = 3;
  public static int DUREE_ASC = 4;
  public static int DUREE_DESC = 5;
  public static int SUCCES_ASC = 6;
  public static int SUCCES_DESC = 7;
  private static ArrayList<ParametreDeTri> _lesParametresDeTri;  
  
  protected int _rang;
  protected int _duree;
  protected boolean _succes;
  protected Calendar _date;

  public Enregistrement(int rang, int duree, boolean succes) {
    _rang = rang;
    _duree = duree;
    _succes = succes;
    _date = Calendar.getInstance();
  }
  
  public Enregistrement(IXMLElement xml) throws Exception {
    this(0,0,true);
		IXMLElement xDate = xml.getFirstChildNamed("date");
		if (xDate != null) {
		  _date = Calendar.getInstance();
		  _date.setTimeInMillis(Long.parseLong(xDate.getContent()));
    } else {
      throw new Exception("La date de l'enregistrement n'est pas indiquée...");
    }    
		IXMLElement xDuree = xml.getFirstChildNamed("duree");
		if (xDuree != null) {
		  _duree = Integer.parseInt(xDuree.getContent());
    } else {
      throw new Exception("La durée de l'enregistrement n'est pas indiquée...");
    }    
    IXMLElement xRang = xml.getFirstChildNamed("rang");
		if (xRang != null) {
		  _rang = Integer.parseInt(xRang.getContent());
    } else {
      throw new Exception("Le rang de l'enregistrement n'est pas indiqué...");
    }
    IXMLElement xSucces = xml.getFirstChildNamed("succes");
		if (xSucces != null) {
		  _succes = (Integer.parseInt(xSucces.getContent()) == 1 ? true : false);
    } else {
      throw new Exception("Le succes de l'enregistrement n'est pas indiqué...");
    }    
  }
  
  public int getRang() {
    return _rang;
  }
  
  public int getDuree() {
    return _duree;
  }
  
 public String toXmlDebut() {
    String xml = "<enregistrement>";
    xml += "<date>" + _date.getTimeInMillis()  + "</date>";
    xml += "<rang>" + getRang()  + "</rang>";
    xml += "<duree>" + getDuree() + "</duree>";
    xml += "<succes>" + (estUnSucces() ? "1" : "0") + "</succes>";
    return xml;
  }
  
  public String toXmlFin() {    
    return "</enregistrement>";
  }
  
  public String toXml() {
    return toXmlDebut() + toXmlFin();
  }
  
  public boolean estUnSucces() {
    return _succes;
  }
  
  public Calendar getDate() {
    return _date;
  }
  
  public static ArrayList<ParametreDeTri> getParametresDeTri() {
    if (_lesParametresDeTri == null) {
      _lesParametresDeTri = new ArrayList<ParametreDeTri>();
      _lesParametresDeTri.add(new ParametreDeTri(DATE_ASC, "Date ancienne->récente"));
      _lesParametresDeTri.add(new ParametreDeTri(DATE_DESC, "Date récente->ancienne"));
      _lesParametresDeTri.add(new ParametreDeTri(RANG_ASC, "Rang croissant"));
      _lesParametresDeTri.add(new ParametreDeTri(RANG_DESC, "Rang décroissant"));
      _lesParametresDeTri.add(new ParametreDeTri(DUREE_ASC, "Durée croissante"));
      _lesParametresDeTri.add(new ParametreDeTri(DUREE_DESC, "Durée décroissante"));
      _lesParametresDeTri.add(new ParametreDeTri(SUCCES_ASC, "Echec->Succès"));
      _lesParametresDeTri.add(new ParametreDeTri(SUCCES_DESC, "Succès->Echec"));
    }
    return _lesParametresDeTri;
  } 

  public static Comparator<Enregistrement> getComparator(ArrayList<ParametreDeTri> paramTri) {
    return new EnregistrementComparator(paramTri);
  }
  
  private static class EnregistrementComparator implements Comparator<Enregistrement> {
    private ArrayList<ParametreDeTri> _parametres;

    private EnregistrementComparator(ArrayList<ParametreDeTri> parametres) {
      _parametres = parametres;
    }

    public int compare(Enregistrement e1, Enregistrement e2) {
      int comparaison = 0;
      for(int i = 0; i < _parametres.size(); i++ ) {
        int idparam = _parametres.get(i).getId();
        if (idparam == Enregistrement.DATE_ASC)
          comparaison = (e1.getDate().getTimeInMillis() < e2.getDate().getTimeInMillis() ? -1 : (e1.getDate().getTimeInMillis() == e2.getDate().getTimeInMillis() ? 0 : 1));
        if (idparam == Enregistrement.DATE_DESC)
           comparaison = (e1.getDate().getTimeInMillis() < e2.getDate().getTimeInMillis() ? 1 : (e1.getDate().getTimeInMillis() == e2.getDate().getTimeInMillis() ? 0 : -1));
        if (idparam == Exercice.RANG_ASC)
          comparaison = (e1.getRang() < e2.getRang() ? -1 : (e1.getRang() == e2.getRang() ? 0 : 1));
        if (idparam == Exercice.RANG_DESC)
          comparaison = (e1.getRang() < e2.getRang() ? 1 : (e1.getRang() == e2.getRang() ? 0 : -1));
        if (idparam == Enregistrement.DUREE_ASC)
          comparaison = (e1.getDuree() < e2.getDuree() ? -1 : (e1.getDuree() == e2.getDuree() ? 0 : 1));
        if (idparam == Enregistrement.DUREE_DESC)
           comparaison = (e1.getDuree() < e2.getDuree() ? 1 : (e1.getDuree() == e2.getDuree() ? 0 : -1));
        if (idparam == Enregistrement.SUCCES_ASC)
          comparaison = (!e1.estUnSucces() && e2.estUnSucces() ? -1 : (!e1.estUnSucces() && !e2.estUnSucces() ? 0 : (e1.estUnSucces() && !e2.estUnSucces() ? 1 : 0)));
        if (idparam == Enregistrement.SUCCES_DESC)
          comparaison = (!e1.estUnSucces() && e2.estUnSucces() ? 1 : (!e1.estUnSucces() && !e2.estUnSucces() ? 0 : (e1.estUnSucces() && !e2.estUnSucces() ? -1 : 0)));          
        if (comparaison != 0) return comparaison;
      }
      return 0;
    }
  }    
    
}
