/*
Copyright Caroline et Serge COUDÉ, (02/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/

package net.sergecoude.TbiGeneral;

import java.util.*;
import net.n3.nanoxml.*;

public class Exercice  {

  private final static long serialVersionUID = 1L;

  public static int DATE_ASC = 0;
  public static int DATE_DESC = 1;
  public static int RANG_ASC = 2;
  public static int RANG_DESC = 3;
  private static ArrayList<ParametreDeTri> _lesParametresDeTri;  
  
  protected Calendar _dateStart;
  protected Calendar _dateStop;
  protected Eleve _eleve;
  protected int _rang;
  protected Vector<Enregistrement> _lesEnregistrements;

  public Exercice(Eleve e, int rang) {
    _lesEnregistrements = new Vector<Enregistrement>();
    _rang = rang;
    _eleve = e;
  }
  
  public Exercice(Eleve eleve, IXMLElement xml) throws Exception {
    this(eleve, 0);
		IXMLElement xDebut = xml.getFirstChildNamed("datedebut");
		if (xDebut != null) {
		  _dateStart = Calendar.getInstance();
		  _dateStart.setTimeInMillis(Long.parseLong(xDebut.getContent()));
    } else {
      throw new Exception("La date de début de l'exercice n'est pas indiquée...");
    }    
		IXMLElement xFin = xml.getFirstChildNamed("datefin");
		if (xFin != null) {
		  _dateStop = Calendar.getInstance();
		  _dateStop.setTimeInMillis(Long.parseLong(xFin.getContent()));
    } else {
      throw new Exception("La date de fin de l'exercice n'est pas indiquée...");
    }    
    IXMLElement xRang = xml.getFirstChildNamed("rang");
		if (xRang != null) {
		  _rang = Integer.parseInt(xRang.getContent());
    } else {
      throw new Exception("Le rang de l'exercice n'est pas indiqué...");
    }    
    IXMLElement xEnregistrements = xml.getFirstChildNamed("enregistrements");
		if (xEnregistrements != null) {
      Vector le = xEnregistrements.getChildrenNamed("enregistrement");
      if (le != null) {
        for(int i = 0; i < le.size(); i++) {
          IXMLElement e = (IXMLElement)(le.elementAt(i));
          getNewEnregistrement(e);
  			}
  		} else 
        throw new Exception("Aucun enregistrement trouvé dans l'exercice...");  			  
    }
  }
  
  public int getRang() {
    return _rang;
  }
  
  public void setSucces(int numExercice) {
  }
  
  public void setEchec(int numExercice) {
  }
  
  public void start() {
    _dateStart = Calendar.getInstance();
  }
  
  public void stop() {
    _dateStop = Calendar.getInstance();
  }
  
  public String toXmlDebut() {
    String xml = "<exercice>";
    xml += "<datedebut>" + _dateStart.getTimeInMillis()  + "</datedebut>";
    xml += "<datefin>" + _dateStop.getTimeInMillis() + "</datefin>";
    xml += "<rang>" + _rang + "</rang>";    
    return xml;
  }
  
  public String toXmlEnregistrements() {
    String xml = "<enregistrements>";
    for(int e = 0; e < _lesEnregistrements.size(); e++)
      xml += _lesEnregistrements.elementAt(e).toXml();
    xml += "</enregistrements>";
    return xml;
  }
  
  public String toXmlFin() {    
    return "</exercice>";
  }
  
  public String toXml() {
    return toXmlDebut() + toXmlEnregistrements() + toXmlFin();
  }
  
  public Enregistrement getEnregistrement() {
    if (_lesEnregistrements.size() > 0)
      return _lesEnregistrements.lastElement();
    return null;
  }
  
  public Enregistrement getEnregistrementByRang(int rang) {
    if (_lesEnregistrements.size() > 0) {
      for(int r = 0; r < _lesEnregistrements.size(); r++) {
        Enregistrement e = _lesEnregistrements.elementAt(r);
        if (e.getRang() == rang)  
          return e;
      }
    }
    return null;
  }
  
  public Vector<Enregistrement> getLesEnregistrements() {
    return _lesEnregistrements;
  }
  
  public Enregistrement getNewEnregistrement(IXMLElement xml) throws Exception {
    _lesEnregistrements.add(new Enregistrement(xml));
    return getEnregistrement();
  }

  public boolean estUnSucces() {
    Enregistrement premier = getEnregistrementByRang(1);
    if (premier != null)
      return premier.estUnSucces();
    return false;
  }
  
  public Calendar getDateStart() {
    return _dateStart;
  }

  public Calendar getDateStop() {
    return _dateStop;
  }
  
  public Eleve getEleve() {
    return _eleve;
  }
  
  public static ArrayList<ParametreDeTri> getParametresDeTri() {
    if (_lesParametresDeTri == null) {
      _lesParametresDeTri = new ArrayList<ParametreDeTri>();
      _lesParametresDeTri.add(new ParametreDeTri(DATE_ASC, "Date ancienne->récente"));
      _lesParametresDeTri.add(new ParametreDeTri(DATE_DESC, "Date récente->ancienne"));
      _lesParametresDeTri.add(new ParametreDeTri(RANG_ASC, "Rang croissant"));
      _lesParametresDeTri.add(new ParametreDeTri(RANG_DESC, "Rang décroissant"));
    }
    return _lesParametresDeTri;
  } 

  public static Comparator<Exercice> getComparator(ArrayList<ParametreDeTri> paramTri) {
    return new ExerciceComparator(paramTri);
  }
  
  private static class ExerciceComparator implements Comparator<Exercice> {
    private ArrayList<ParametreDeTri> _parametres;

    private ExerciceComparator(ArrayList<ParametreDeTri> parametres) {
      _parametres = parametres;
    }

    public int compare(Exercice e1, Exercice e2) {
      int comparaison = 0;
      for(int i = 0; i < _parametres.size(); i++ ) {
        int idparam = _parametres.get(i).getId();
        if (idparam == Exercice.DATE_ASC)
          comparaison = (e1.getDateStart().getTimeInMillis() < e2.getDateStart().getTimeInMillis() ? -1 : (e1.getDateStart().getTimeInMillis() == e2.getDateStart().getTimeInMillis() ? 0 : 1));
        if (idparam == Exercice.DATE_DESC)
           comparaison = (e1.getDateStart().getTimeInMillis() < e2.getDateStart().getTimeInMillis() ? 1 : (e1.getDateStart().getTimeInMillis() == e2.getDateStart().getTimeInMillis() ? 0 : -1));
        if (idparam == Exercice.RANG_ASC)
          comparaison = (e1.getRang() < e2.getRang() ? -1 : (e1.getRang() == e2.getRang() ? 0 : 1));
        if (idparam == Exercice.RANG_DESC)
          comparaison = (e1.getRang() < e2.getRang() ? 1 : (e1.getRang() == e2.getRang() ? 0 : -1));
        if (comparaison != 0) return comparaison;
      }
      return 0;
    }
  }    
      
}
