/*
Copyright Caroline et Serge COUDÉ, (01/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/
package net.sergecoude.TbiGeneral;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.awt.image.*;
import java.awt.geom.*;

public class Mire extends JPanel implements ComponentListener  {

  private final static long serialVersionUID = 1L;

  private int _nbExercicesCorrects = 0;
  private int _nbExercices = 0;
  private Color _couleurRemplissage = new Color(100,100,255,255);
	
  public Mire() {
    setOpaque(true);
    addComponentListener(this);
    setNbExercicesCorrects(0);
    setNbExercices(0);   
  }
  
  public void componentHidden(ComponentEvent e) {
  }

  public void componentMoved(ComponentEvent e) {
  }
  
  public void componentResized(ComponentEvent e) {
    Dimension d = getSize();
    Dimension dMin = getMinimumSize();
    Dimension dMax = getMaximumSize();
    boolean modifSize = false;
    if (d.width < dMin.width) {
      d.width = dMin.width;
      modifSize = true;
    }
    if (d.width > dMax.width) {
      d.width = dMax.width;
      modifSize = true;
    }
    if (d.height < dMin.height) {
      d.height = dMin.height;
      modifSize = true;
    }
    if (d.height > dMax.height) {
      d.height = dMax.height;
      modifSize = true;
    }
    if (modifSize) {
      setSize(d);
      return;
    }      
    boolean correctAspectRatio = (d.width == d.height); 
    if (!correctAspectRatio) {
      final JPanel _this = this;
      final int _width = d.width > d.height ? d.height : d.width; 
      final int _height = _width;
      SwingUtilities.invokeLater(new Runnable(){
        public void run() {
          _this.setSize(_width, _height);
        }
      });
    }
  }

  public void componentShown(ComponentEvent e) {
  }

  public BufferedImage getBufferedImage(int w, int h) {
    setSize(w,h);
    BufferedImage buf = new BufferedImage(w, h, BufferedImage.TYPE_BYTE_GRAY/*TYPE_INT_RGB*/);
    Graphics2D g = buf.createGraphics();
    paint(g);
    return buf;
  }

  public void paint(Graphics og) {
    Graphics2D g = (Graphics2D)og;
    Dimension dim = getSize();
    g.setPaint(Color.white);
    g.fillRect(0,0,dim.width,dim.height);
    g.setPaint(Color.black);
    if (_nbExercices > 0) {
  		g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
      g.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      if (dim.width >= 120)
        g.setStroke(new BasicStroke(3));
      else if (dim.width >= 80)
        g.setStroke(new BasicStroke(2));
      else 
        g.setStroke(new BasicStroke(1));
      int dInter = (dim.width / 10) / 2; // 10 exercices maximum
      float rD = dInter * _nbExercicesCorrects;
      Ellipse2D.Float disque = new Ellipse2D.Float((float)(dim.width/2-rD), (float)(dim.height/2-rD), rD*2, rD*2);
      g.setPaint(_couleurRemplissage);
      g.fill((Shape)disque);
      g.setPaint(Color.black);
      float rC = dInter;
      Ellipse2D.Float cercle = new Ellipse2D.Float((float)(dim.width/2-rC), (float)(dim.height/2-rC), rC*2, rC*2);
      for(int i = 0; i < _nbExercices; i++) {
        g.draw((Shape)cercle);
        rC += dInter;
        cercle.setFrame((float)(dim.width/2-rC), (float)(dim.height/2-rC), rC*2, rC*2);
      }
    }
  }
  
  public void setNbExercicesCorrects(int n) {
    _nbExercicesCorrects = n;
    repaint();
  }  

  public void setNbExercices(int n) {
    _nbExercices = n;
    repaint();
  } 
}


