/*
Copyright Caroline et Serge COUDÉ, (02/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/

package net.sergecoude.TbiGeneral;

import java.util.*;
import java.io.*;
import net.sergecoude.xml.*;
import javax.swing.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.text.*;

public class TbiConfGenerale {

  private final static long serialVersionUID = 1L;
	
	private TreeMap<String,HashMap<String,HashMap<String,String>>> _configuration;
	public static int MESSAGE_ERREUR = 1;
	public static int MESSAGE_INFORMATION = 2;
	public static int MESSAGE_AVERTISSEMENT = 3;
	private String _fichierConf = "TbiGeneral.conf";
	private File _fileFichierConf;

	public TbiConfGenerale() {
	}

	public String getConfiguration(String param) {
	  Set lesFichiers = _configuration.keySet();
    Iterator it = lesFichiers.iterator();
    String val = null;
    while (it.hasNext()){
      String rang = (String)it.next();
//      System.out.println("    je recherche " + param + " dans " + rang);
      String valtmp = getConfiguration(rang, param);
      if (valtmp != null && !valtmp.equals(""))  
        val = valtmp;
    }
//    System.out.println("j'ai trouve : " + val); 
    return val;
	}

	public String getConfiguration(String rang, String param) {
//	  System.out.print("Je recherche " + param + " dans le fichier de rang " + rang + " : ");
    String val = null;
    HashMap<String,HashMap<String,String>> confFichier = _configuration.get(rang);
    if (confFichier != null) {
      HashMap<String,String> conf = confFichier.get("conf");
      if (conf != null) {
        val = conf.get(param);
      } 
    }
//    System.out.println(val);  
    return val;
	}
	
	public String getFichierConfigurationGenerale() {
    try {
        return _fileFichierConf.toURI().toURL().toString();
    } catch (MalformedURLException mfue) {
    }
    return _fichierConf;
  }
  
	public boolean setConfiguration(String rang, String param, String valeur) {
    HashMap<String,HashMap<String,String>> confFichier = _configuration.get(rang);
    if (confFichier != null) {
      HashMap<String,String> conf = confFichier.get("conf");
      conf.put(param, valeur);
      return true;
    }
    return false;
	}
	
	public boolean chargeConfiguration(String rang, String fichier, boolean silencieux, boolean oblige) {
//	  System.out.println("Je charge la conf contenue dans " + fichier);
    return chargeConfiguration(rang, fichier, getClass().getResource("/" + fichier), silencieux, true) &&
           chargeConfiguration(rang, new File(fichier), silencieux, oblige);
  }  

  public boolean chargeConfiguration(String rang, File fichier, boolean silencieux, boolean oblige) {
//    System.out.println("Je charge le fichier " + fichier);
    if (fichier.exists()) {
      try {
        return chargeConfiguration(rang, fichier.getAbsolutePath(), fichier.toURI().toURL(), silencieux, oblige);
      } catch (MalformedURLException mfue) {
        if (!silencieux && oblige) { 
          message("\nLe chemin du fichier de configuration " + fichier.getAbsolutePath() + " n'est pas valide...", MESSAGE_INFORMATION);
          return false;
        }
      }
    } else if (oblige) {
      if (!silencieux)
        message("\nLe fichier de configuration " + fichier.getAbsolutePath() + " est introuvable ou non accessible...\n",MESSAGE_ERREUR);
      return false; 
    }
    return true;
  }
    
	public boolean chargeConfiguration(String rang, String cheminFichier, URL urlConf, boolean silencieux, boolean oblige) {
//	  System.out.println("Je charge l'URL " + urlConf + " correspondant a " + cheminFichier + " au rang " + rang);
		LinkedList laliste = new LinkedList();
		Parseur XMLFile = null;
		ArrayList le = null;
		if (_configuration == null)
      _configuration = new TreeMap<String,HashMap<String,HashMap<String,String>>>();
    if (urlConf == null) {
      if (oblige) {
        if (!silencieux)
          message("\nLe fichier de configuration '" + cheminFichier + "' est introuvable ou non accessible...\n",MESSAGE_ERREUR);
        return false;
      } else
        return true;
    } 
    XMLFile = new Parseur(urlConf);
    if (!XMLFile.isValid()) {
      if (!silencieux)
        message("\nLe fichier de configuration " + urlConf + " n'est pas valide...\n",MESSAGE_ERREUR);
      return false;
    }
    le = XMLFile.getEnfant("configuration");
    String fichier = urlConf.toString();
    HashMap<String,HashMap<String,String>> confFichier = _configuration.get(rang);
    HashMap<String,String> conf = null;
    HashMap<String,String> pos = null;
    HashMap<String,String> chemin = null;
    if (confFichier == null) {
      confFichier = new HashMap<String,HashMap<String,String>>();
      conf = new HashMap<String,String>();
      pos = new HashMap<String,String>();
      chemin = new HashMap<String,String>();
      chemin.put("chemin", cheminFichier);
      confFichier.put("conf", conf);
      confFichier.put("pos", pos);
      confFichier.put("chemin", chemin);
      _configuration.put(rang, confFichier);
    } else {
      conf = confFichier.get("conf");
      pos = confFichier.get("pos");
      chemin = confFichier.get("chemin");
      chemin.put("chemin", cheminFichier); 
    }
    NumberFormat formatter = new DecimalFormat("000");
    if (le != null) {
      for(int i=0; i < le.size(); i++) {
        XMLElement e = (XMLElement)le.get(i);
        Set sRangs = pos.keySet();
        Iterator itRang = sRangs.iterator();
        boolean valeurDejaPresente = false;
        while (itRang.hasNext()) {
          String sNomConf = pos.get((String)itRang.next());
          if (sNomConf.equals(e.getNom())) {
            valeurDejaPresente = true;
            break;
          }
        }
        conf.put(e.getNom(),e.getValeur()); // on écrase si déjà présente
        if (!valeurDejaPresente)
          pos.put(formatter.format(i+1), e.getNom());
      }
    }
    return true;
  }

	public void message(String m, int t) {
    if (t == MESSAGE_ERREUR) 
      JOptionPane.showMessageDialog(null, m, "Message", JOptionPane.ERROR_MESSAGE);
    if (t == MESSAGE_AVERTISSEMENT) 
      JOptionPane.showMessageDialog(null, m, "Message", JOptionPane.WARNING_MESSAGE);
    if (t == MESSAGE_INFORMATION) 
      JOptionPane.showMessageDialog(null, m, "Message", JOptionPane.INFORMATION_MESSAGE);
  }
  
  public boolean sauveConfiguration(String rangConf) {
    HashMap<String,HashMap<String,String>> confFichier = _configuration.get(rangConf);
    if (confFichier == null) 
      return false;
    HashMap<String,String> chemin = confFichier.get("chemin");
    HashMap<String,String> conf = confFichier.get("conf");
    HashMap<String,String> pos = confFichier.get("pos");
    File fichier = new File(chemin.get("chemin")); 
    try {
      PrintWriter out  = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(fichier),"UTF-8")));
      java.util.List<String> lRangs = new ArrayList<String>();
      Set sRangs = pos.keySet();
      Iterator itRang = sRangs.iterator();
      while (itRang.hasNext()) {
        String rang = (String)itRang.next();
        lRangs.add(rang);
      }
      Collections.sort(lRangs);
      out.println("<configuration>");
      itRang = lRangs.iterator();
      while(itRang.hasNext()){
        String rang = (String)itRang.next();
        String param = pos.get(rang);
        String valeur = conf.get(param);
        out.println("\t<" + param + ">" + valeur + "</" + param + ">");
      }        
      out.println("</configuration>");  
      out.close();
      lRangs = null;
      return true;
    } catch(Exception e) {
      Log.logStatic(e);
    }
    return false;
  }
  
  public String toString() {
	  Set lesFichiers = _configuration.keySet();
    Iterator it = lesFichiers.iterator();
    String val = "";
    while (it.hasNext()){
      String fichier = (String)it.next();
      val += "\n\t" + fichier + ": \n";
      HashMap<String,HashMap<String,String>> confFichier = _configuration.get(fichier);
      val += confFichier.toString(); 
    } 
    return val;  
  }
  	
}
