/*
Copyright Caroline et Serge COUDÉ, (04/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/
package net.sergecoude.TbiRituel;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import net.sergecoude.TbiGeneral.*;

public class Element extends bouton {

  private final static long serialVersionUID = 1L;
  
	protected TbiRituel _app;
	protected String _nom;
	protected URL _urlNom;
	protected Point _spotMouse = null;
	protected Point _posIni = null;
	protected Dimension _dimIni = null;
	protected boolean _coche = false;
	protected boolean _deplaceable = true;
	protected ByteArrayInputStream _rawAudio = null;
	protected String _id = "";

	public Element(TbiRituel app, String nom, String id, URL ressourceImage, URL ressourceNom) throws Exception { 
    super(ressourceImage);
		_app = app;
		_nom = nom;
    _urlNom = ressourceNom;
    _id = id;
	}
			
	public Element(TbiRituel app, String nom, URL ressourceImage, URL ressourceNom) throws Exception { 
    this(app, nom, nom, ressourceImage, ressourceNom);
	}
	
	public Element(TbiRituel app, String nom,URL ressourceNom) throws Exception { 
	  super(nom);
		_app = app;
		_nom = nom;
    _urlNom = ressourceNom;
	}
	
	public Element(TbiRituel app, ItemEleve iEleve) throws Exception { 
	  super(toBufferedImage(iEleve, app.getDimensionPhoto().width, (int)(app.getDimensionPhoto().height/ItemEleve.ratio)));
		_app = app;
		_nom = iEleve.getEleve().getNomUsuel();
    _urlNom = null;
    _rawAudio = iEleve.getAudio();
    _id = iEleve.getEleve().getId();
	}
	
  public static BufferedImage toBufferedImage(ItemEleve iEleve, int w, int h) {
    Image image = iEleve.getImage();
    boolean hasAlpha = true;
    if (image != null) {
      image = new ImageIcon(image).getImage();
      hasAlpha = hasAlpha(image);
    }
    BufferedImage bimage = null;
    GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
    try {
      int transparency = Transparency.OPAQUE;
      if (hasAlpha) 
        transparency = Transparency.BITMASK;
      GraphicsDevice gs = ge.getDefaultScreenDevice();
      GraphicsConfiguration gc = gs.getDefaultConfiguration();
      bimage = gc.createCompatibleImage( w, h, transparency);
    } catch (HeadlessException e) {
    }
    if (bimage == null) {
      int type = BufferedImage.TYPE_INT_RGB;
      if (hasAlpha) 
        type = BufferedImage.TYPE_INT_ARGB;
      bimage = new BufferedImage(w, h, type);
    }
    Graphics2D g = bimage.createGraphics();
    if (image != null)
      g.drawImage(image, 0, 0, w, w, null);
    else { // si l'image n'existe pas, alors une croix noire
      Stroke oldStroke = g.getStroke();
      g.setStroke(new BasicStroke(5));
      g.setColor(Color.WHITE);
      g.fillRect(0,0,w,w);
      g.setColor(Color.BLACK);
      g.drawLine(0,0,w,w);
      g.drawLine(w,0,0,w);
      g.setStroke(oldStroke);
    }
    g.setColor(Color.WHITE);
    g.fillRect(0,w,w,h);
    g.setFont(new Font("SansSerif",Font.BOLD,12));
    g.setColor(Color.BLACK);
    Dimension fontSize = foundFontSize(g, iEleve.getEleve().getNomUsuel(), w, h-w);
    g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
    g.drawString(iEleve.getEleve().getNomUsuel(),(w-fontSize.width)/2,h-g.getFontMetrics().getDescent());
    g.dispose();
    return bimage;
  }
  
  public static Dimension foundFontSize(Graphics2D g, String chaine, int largeur, int hauteur) {
    int l = 0, h = 0, s = 0;
    while(l <= largeur && h <= hauteur) {
      s++;
      g.setFont(g.getFont().deriveFont((float)s));
      FontMetrics fontMetrics = g.getFontMetrics();
      l = fontMetrics.stringWidth(chaine);
      h = fontMetrics.getAscent()+fontMetrics.getDescent();
    }
    g.setFont(g.getFont().deriveFont((float)(s-1)));
    FontMetrics fontMetrics = g.getFontMetrics();
    l = fontMetrics.stringWidth(chaine);
    h = fontMetrics.getAscent()+fontMetrics.getDescent();
    return new Dimension(l, h);
  } 
  
  public static boolean hasAlpha(Image image) {
    if (image instanceof BufferedImage) {
        BufferedImage bimage = (BufferedImage)image;
        return bimage.getColorModel().hasAlpha();
    }
    PixelGrabber pg = new PixelGrabber(image, 0, 0, 1, 1, false);
    try {
        pg.grabPixels();
    } catch (InterruptedException e) {
    }
    ColorModel cm = pg.getColorModel();
    return cm.hasAlpha();  
  }

  public String toString() {
    return _nom;
  }

  public String getNom() {
    return _nom;
  }
  
  public URL getURLNom() {
    return _urlNom;
  } 
  
  public void setId(String id) {
    _id = id;
  }
  
  public String getId() {
    return _id;
  }  
  
  public ByteArrayInputStream getAudio() {
    return _rawAudio;
  }
  
	public void processMouseEvent(MouseEvent e) {
	  super.processMouseEvent(e);
		switch(e.getID()) {
		case MouseEvent.MOUSE_PRESSED:
		  _spotMouse = e.getPoint();
			break;
		case MouseEvent.MOUSE_RELEASED:
			break;
		case MouseEvent.MOUSE_ENTERED:
			break;
		case MouseEvent.MOUSE_EXITED:
			break;
		}
	}
  
  public Point getSpotMouse() {
    return _spotMouse;
  }  
  
  public void setIniLocation(int x, int y) {
    if (_posIni == null) {
      _posIni = new Point(x,y);
    } else {
      _posIni.x = x;
      _posIni.y = y;
    }
    super.setLocation(x,y);
  }

  public void setLocation(int x, int y) {
    if (_deplaceable)
      super.setLocation(x,y);
  }
  
  public Point getIniLocation() {
    return _posIni;
  } 
	
  public boolean contient(Element e) {
    Rectangle rThis = getBounds();
    Point pThis = getLocationOnScreen();
    rThis.x = pThis.x;
    rThis.y = pThis.y;
    Rectangle rElem = e.getBounds();
    Point pElem = e.getLocationOnScreen();
    rElem.x = pElem.x;
    rElem.y = pElem.y;
    return rThis.contains(rElem);
  }

  public boolean contient(int x, int y) {
    Rectangle rThis = getBounds();
    Point pThis = getLocationOnScreen();
    rThis.x = pThis.x;
    rThis.y = pThis.y;
    return rThis.contains(x, y);
  }
  
  public void setCoche(boolean oui) {
    _coche = oui;
    repaint();
  }

  public boolean estCoche() {
    return _coche;
  }
  
  public void setDeplaceable(boolean oui) {
    _deplaceable = oui;
  }

  public boolean estDeplaceable() {
    return _deplaceable;
  }
  
  public void paint(Graphics g) {
		super.paint(g);
		if (_coche) {
		  Graphics2D g2d = (Graphics2D)g;
		  g2d.setColor(getForeground());
      float dx = ((float)largeur*0.1f);
      g2d.setStroke(new BasicStroke(dx));
      Shape circle = new Ellipse2D.Float(dx, dx, (float)largeur-2*dx, (float)hauteur-2*dx);
      g2d.draw(circle);      
    }
	}

	public void setIniDimension(int l, int h) {
    if (_dimIni == null)
      _dimIni = new Dimension(0, 0);
    _dimIni.width = l;
    _dimIni.height = h;
    setDimension(l, h);
  }
  
  public Dimension getIniDimension() {
    return _dimIni;
  }
  
  public void setRessourceImage(URL ressourceImage) throws Exception { 
	  setImage(ressourceImage);
	}
  
  public void setRessourceAudio(URL ressourceAudio) { 
    _urlNom = ressourceAudio;
  }   

}


