/*
Copyright Caroline et Serge COUDÉ, (04/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/

package net.sergecoude.TbiRituel;

import java.util.*;
import java.io.*;
import net.n3.nanoxml.*;
import net.sergecoude.TbiGeneral.*;

public class EvaluationRituel extends Evaluation  {

  private final static long serialVersionUID = 1L;
  
  private String _temperature; // ne pas initialiser dans la déclaration
  private String _ciel; // ne pas initialiser dans la déclaration
  private String _precipitation; // ne pas initialiser dans la déclaration
  private String _vent; // ne pas initialiser dans la déclaration
  private String _elevesMaison; // ne pas initialiser dans la déclaration
  private String _elevesEcole; // ne pas initialiser dans la déclaration
  
  public EvaluationRituel(TbiRituel app) {
    super(app);
  }
  
  public EvaluationRituel(TbiRituel app, String chemin) throws Exception {
    super(app, chemin);      
  }

  public EvaluationRituel(TbiApp app) {
    super(app);
  }
  
  public EvaluationRituel(TbiApp app, String chemin) throws Exception {
    super(app, chemin);
  }
 
  @Override
  public void chargeEvaluation(IXMLElement xml) throws Exception {
		IXMLElement xNom = xml.getFirstChildNamed("nom");
		if (xNom != null) {
		  setNom(xNom.getContent());
    } else 
      throwNewException("Le nom de l'évaluation n'est pas indiqué... [" + _chemin + "]");
		IXMLElement xCle = xml.getFirstChildNamed("cle");
		if (xCle != null) {
		  setCle(xCle.getContent());
    } else 
      throwNewException("La clé de l'évaluation n'est pas indiquée... [" + _chemin + "]");
		IXMLElement xDebut = xml.getFirstChildNamed("datedebut");
		if (xDebut != null) {
		  Calendar date = Calendar.getInstance();
		  date.setTimeInMillis(Long.parseLong(xDebut.getContent()));
		  setDateStart(date);
    } else 
      throwNewException("La date de début d'évaluation n'est pas indiquée... [" + _chemin + "]");
  	IXMLElement xFin = xml.getFirstChildNamed("datefin");
		if (xFin != null) {
		  Calendar date = Calendar.getInstance();
		  date.setTimeInMillis(Long.parseLong(xFin.getContent()));
		  setDateStop(date);
    } else
      throwNewException("La date de fin d'évaluation n'est pas indiquée... [" + _chemin + "]");
    IXMLElement xClasse = xml.getFirstChildNamed("classe");
		if (xClasse != null) {
		  _classe = _app.getClasseById(xClasse.getContent());
		  if (_classe == null) 
		    throwNewException("La classe de l'évaluation n'est pas présente... [" + _chemin + "]");
    }
    IXMLElement xVersion = xml.getFirstChildNamed("version");
		if (xVersion != null) {
		  setVersionApplication(xVersion.getContent());
    }
		IXMLElement xLangue = xml.getFirstChildNamed("langue");
		if (xLangue != null) {
		  setLangue(xLangue.getContent());
    }               
		IXMLElement xTemperature = xml.getFirstChildNamed("temperature");
		if (xTemperature != null) {
		  setTemperature(xTemperature.getContent());
    }
		IXMLElement xCiel = xml.getFirstChildNamed("ciel");
		if (xCiel != null) {
		  setCiel(xCiel.getContent());
    }
		IXMLElement xPrecipitation = xml.getFirstChildNamed("precipitation");
		if (xPrecipitation != null) {
		  setPrecipitation(xPrecipitation.getContent());
    }
 		IXMLElement xVent = xml.getFirstChildNamed("vent");
		if (xVent != null) {
		  setVent(xVent.getContent());
    }
		IXMLElement xElevesMaison = xml.getFirstChildNamed("elevesmaison");
		if (xElevesMaison != null) {
		  setElevesMaison(xElevesMaison.getContent());
    }
		IXMLElement xElevesEcole = xml.getFirstChildNamed("elevesecole");
		if (xElevesEcole != null) {
		  setElevesEcole(xElevesEcole.getContent());
    }
    _estEnregistree = true;
  }
  
  public void setTemperature(String temperature) {
    _temperature = temperature;
  }
  
  public String getTemperature() {
    return _temperature;
  }

  public void setCiel(String ciel) {
    _ciel = ciel;
  }
  
  public String getCiel() {
    return _ciel;
  }

  public void setPrecipitation(String precipitation) {
    _precipitation = precipitation;
  }
  
  public String getPrecipitation() {
    return _precipitation;
  }

  public void setVent(String vent) {
    _vent = vent;
  }
  
  public String getVent() {
    return _vent;
  }
  
  public void setElevesMaison(String eleves) {
    _elevesMaison = eleves;
  }
  
  public String getElevesMaison() {
    return _elevesMaison;
  }  

  public void setElevesEcole(String eleves) {
    _elevesEcole = eleves;
  }
  
  public String getElevesEcole() {
    return _elevesEcole;
  }  
  
  @Override
  public boolean contientDesEnregistrements() {
    return true;  
  }
  
  public boolean contientMeteo() {
    return ((getTemperature() != null && !getTemperature().equals("")) ||
        (getCiel() != null && !getCiel().equals("")) ||
        (getPrecipitation() != null && !getPrecipitation().equals("")) ||
        (getVent() != null && !getVent().equals("")));
  }  

  public boolean contientPresence() {
    return ((getElevesEcole() != null && !getElevesEcole().equals("")) ||
        (getElevesMaison() != null && !getElevesMaison().equals("")));
  }  
  
  public String toXmlDebut() {
    String xml = "";
    if (_app.getClasse() == null) {
      xml = "<evaluation>";
      xml += "<nom>" + _nom + "</nom>";
      xml += "<cle>" + _cle + "</cle>";
      xml += "<datedebut>" + _dateStart.getTimeInMillis() + "</datedebut>";
      xml += "<datefin>" + _dateStop.getTimeInMillis() + "</datefin>";
      xml += "<version>" + _versionApplication + "</version>";
      xml += "<langue>" + (getLangue() != null ? getLangue() : "") + "</langue>";
    }
    if (getTemperature() != null && !getTemperature().equals(""))
      xml += "<temperature>" + getTemperature() + "</temperature>";
    if (getCiel() != null && !getCiel().equals(""))
      xml += "<ciel>" + getCiel() + "</ciel>";
    if (getPrecipitation() != null && !getPrecipitation().equals(""))
      xml += "<precipitation>" + getPrecipitation() + "</precipitation>";
    if (getVent() != null && !getVent().equals(""))
      xml += "<vent>" + getVent() + "</vent>";
    if (getElevesEcole() != null && !getElevesEcole().equals(""))
      xml += "<elevesecole>" + getElevesEcole() + "</elevesecole>";
    if (getElevesMaison() != null && !getElevesMaison().equals(""))
      xml += "<elevesmaison>" + getElevesMaison() + "</elevesmaison>";
    if (_app.getClasse() == null) 
      return xml;
    else 
      return super.toXmlDebut() + xml;
  }
   
}
