/*
Copyright Caroline et Serge COUDÉ, (04/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/
package net.sergecoude.TbiRituel;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.net.*;
import java.awt.image.*;
import javax.imageio.*;
import net.sergecoude.TbiGeneral.*;

public class PanelTableauMeteo extends PanelJeuMulti {

  private final static long serialVersionUID = 1L;
  
  private Element _eMeteo; // titre
  private Element _eTTresChaud, _eTChaud, _eTTempere, _eTFroid, _eTTresFroid; // température
  private Element _eCCouvert, _eCEnsoleille, _eCEnsoleilleBlanc, _eCOrage, _eCBrouillard; // ciel
  private Element _ePSec, _ePPluie, _ePAverse, _ePGrele, _ePNeige; // précipitation
  private Element _eVNul,  _eVFort, _eVTempete; // vent
  private Element _eSTemperature, _eSCiel, _eSPrecipitation, _eSVent; // saisies
  private Element _elementSelectionne = null; 
  private int _dyEtiquette = 10; // nb de pixels entre deux rangées d'étiquettes
  private int _dxEtiquette = 5; // nb de pixels entre deux étiquettes 
  private int _oldZOrder = 3000;
  private int _mouseX;
  private int _mouseY;
  private Element _eSelTemperature = null;
  private Element _eSelCiel = null;
  private Element _eSelPrecipitation = null;
  private Element _eSelVent = null;

  public PanelTableauMeteo(TbiRituel app) {
    super(app);

		_pMain.setLayout(null); // définition à faire
		// tous les éléments du plateau sont à insérer dans _pMain
  }
  
  @Override
  public void init() {
    if (_init) {
      repaint();
      return;
    }
    centreBarreProgression();
    _bProgression.setBiColors(new Color(128,200,128), Color.white);
    _bProgression.setProgression(0);
    repaint();   
    (new Thread(new Runnable() {
      public void run() {
        float nbElements = 0;
        float nbElementsTotal = 23;
        try {
          _eMeteo = new Element((TbiRituel)_app, "La météo du jour", "Meteo", _app.getSkin().getRessource("imgMeteo"), _app.getSkin().getRessource("audMeteo"));
          _pMain.add(_eMeteo);
          _eMeteo.addActionListener(getThis());
          _eMeteo.setDeplaceable(false);
  		    nbElements++;
	   	    _bProgression.setProgression((int)(100f*nbElements/nbElementsTotal));
        } catch(Exception e) {
          _app.log(e);
        }
        nbElements = creeLesTemperatures((TbiRituel)_app, nbElements, nbElementsTotal);
        nbElements = creeLesCieux((TbiRituel)_app, nbElements, nbElementsTotal);
        nbElements = creeLesPrecipitations((TbiRituel)_app, nbElements, nbElementsTotal);
        nbElements = creeLesVents((TbiRituel)_app, nbElements, nbElementsTotal);
        nbElements = creeZoneSaisie((TbiRituel)_app, nbElements, nbElementsTotal);
        _pEvaluations.setVisible(false);
        _bProgression.setProgression(100);
    	  _app.majAffichage();
        setSouris(true);
        agenceLesComposants();
        SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            retireBarreProgression();
            _init = true;
          }
        });
      };
    })).start();    
  }  
  
  private float creeLesTemperatures(TbiRituel app, float cpt, float cptTotal) {
    try {
      _eTTresChaud = new Element(app, "Très chaud", "TresChaud",  _app.getSkin().getRessource("imgTresChaud"), _app.getSkin().getRessource("audTresChaud"));
      _pMain.add(_eTTresChaud);
      _eTTresChaud.addActionListener(this);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _eTChaud = new Element(app, "Chaud", "Chaud", _app.getSkin().getRessource("imgChaud"), _app.getSkin().getRessource("audChaud"));
      _pMain.add(_eTChaud);
      _eTChaud.addActionListener(this);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _eTTempere = new Element(app, "Doux", "Tempere", _app.getSkin().getRessource("imgTempere"), _app.getSkin().getRessource("audTempere"));
      _pMain.add(_eTTempere);
      _eTTempere.addActionListener(this);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _eTFroid = new Element(app, "Froid", "Froid", _app.getSkin().getRessource("imgFroid"), _app.getSkin().getRessource("audFroid"));
      _pMain.add(_eTFroid);
      _eTFroid.addActionListener(this);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _eTTresFroid = new Element(app, "Très froid", "TresFroid", _app.getSkin().getRessource("imgTresFroid"), _app.getSkin().getRessource("audTresFroid"));
      _pMain.add(_eTTresFroid);
      _eTTresFroid.addActionListener(this);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      return cpt;      
    } catch(Exception e) {
      app.log(e);
      return cpt;
    }
  }
  
  private float creeLesCieux(TbiRituel app, float cpt, float cptTotal) {
    try {
      _eCCouvert = new Element(app, "Couvert", "Couvert", _app.getSkin().getRessource("imgCouvert"), _app.getSkin().getRessource("audCouvert"));
      _pMain.add(_eCCouvert);
      _eCCouvert.addActionListener(this);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _eCEnsoleille = new Element(app, "Ensoleillé", "Ensoleille", _app.getSkin().getRessource("imgEnsoleille"), _app.getSkin().getRessource("audEnsoleille"));
      _pMain.add(_eCEnsoleille);
      _eCEnsoleille.addActionListener(this);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _eCEnsoleilleBlanc = new Element(app, "Ensoleille avec nuages blancs", "EnsoleilleBlanc", _app.getSkin().getRessource("imgEnsoleilleBlanc"), _app.getSkin().getRessource("audEnsoleilleBlanc"));
      _pMain.add(_eCEnsoleilleBlanc);
      _eCEnsoleilleBlanc.addActionListener(this);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _eCOrage = new Element(app, "Orageux", "Orage", _app.getSkin().getRessource("imgOrage"), _app.getSkin().getRessource("audOrage"));
      _pMain.add(_eCOrage);
      _eCOrage.addActionListener(this);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _eCBrouillard = new Element(app, "Brouillard", "Brouillard", _app.getSkin().getRessource("imgBrouillard"), _app.getSkin().getRessource("audBrouillard"));
      _pMain.add(_eCBrouillard);
      _eCBrouillard.addActionListener(this);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      return cpt;      
    } catch(Exception e) {
      app.log(e);
      return cpt;
    }
  }
  
  private float creeLesPrecipitations(TbiRituel app, float cpt, float cptTotal) {
    try {
      _ePSec = new Element(app, "Pas de pluie", "Sec", _app.getSkin().getRessource("imgSec"), _app.getSkin().getRessource("audSec"));
      _ePSec.addActionListener(this);
      _pMain.add(_ePSec);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _ePPluie = new Element(app, "Pluie", "Pluie", _app.getSkin().getRessource("imgPluie"), _app.getSkin().getRessource("audPluie"));
      _ePPluie.addActionListener(this);
      _pMain.add(_ePPluie);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _ePAverse = new Element(app, "Averse", "Averse", _app.getSkin().getRessource("imgAverse"), _app.getSkin().getRessource("audAverse"));
      _ePAverse.addActionListener(this);
      _pMain.add(_ePAverse);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _ePGrele = new Element(app, "Grêle", "Grele", _app.getSkin().getRessource("imgGrele"), _app.getSkin().getRessource("audGrele"));
      _ePGrele.addActionListener(this);
      _pMain.add(_ePGrele);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _ePNeige = new Element(app, "Neige", "Neige", _app.getSkin().getRessource("imgNeige"), _app.getSkin().getRessource("audNeige"));
      _ePNeige.addActionListener(this);
      _pMain.add(_ePNeige);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      return cpt; 
    } catch(Exception e) {
      app.log(e);
      return cpt;
    }
  }
  
  private float creeLesVents(TbiRituel app, float cpt, float cptTotal) {
    try {
      _eVNul = new Element(app, "Pas de vent", "Nul", _app.getSkin().getRessource("imgNul"), _app.getSkin().getRessource("audNul"));
      _pMain.add(_eVNul);
      _eVNul.addActionListener(this);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _eVFort = new Element(app, "Fort", "Fort", _app.getSkin().getRessource("imgFort"), _app.getSkin().getRessource("audFort"));
      _pMain.add(_eVFort);
      _eVFort.addActionListener(this);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _eVTempete = new Element(app, "Tempête", "Tempete", _app.getSkin().getRessource("imgTempete"), _app.getSkin().getRessource("audTempete"));
      _pMain.add(_eVTempete);
      _eVTempete.addActionListener(this);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      return cpt;      
    } catch(Exception e) {
      app.log(e);
      return cpt;
    }
  }
  
  private void initElementSaisie(Element e) {
    _pMain.add(e);
    e.addActionListener(this);
    e.setDeplaceable(false);
  }

  private float creeZoneSaisie(TbiRituel app, float cpt, float cptTotal) {
    try {
      _eSTemperature = new Element(app, "Température", "Temperature", _app.getSkin().getRessource("imgTemperature"), _app.getSkin().getRessource("audTemperature"));
      initElementSaisie(_eSTemperature);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _eSCiel = new Element(app, "Ciel", "Ciel", _app.getSkin().getRessource("imgCiel"), _app.getSkin().getRessource("audCiel"));
      initElementSaisie(_eSCiel);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _eSPrecipitation = new Element(app, "Précipitation", "Precipitation", _app.getSkin().getRessource("imgPrecipitation"), _app.getSkin().getRessource("audPrecipitation"));
      initElementSaisie(_eSPrecipitation);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
      _eSVent = new Element(app, "Vent", "Vent", _app.getSkin().getRessource("imgVent"), _app.getSkin().getRessource("audVent"));
      initElementSaisie(_eSVent);
      cpt++;
	    _bProgression.setProgression((int)(100f*cpt/cptTotal));
	    return cpt;
    } catch(Exception e) {
      app.log(e);
      return cpt;
    }
  }
  
  public void actionPerformed(ActionEvent evt) {
		Object source = evt.getSource();
		if (source instanceof Element) {
      Element elem = (Element)source;
      if (((TbiRituel)_app).getAvecNomPrononce() && elem != _elementSelectionne) {
        try {
          _app.play(elem.getURLNom().toURI().toURL());
        } catch (MalformedURLException mue) {
          _app.message(mue.toString(), _app.MESSAGE_ERREUR);
        } catch (URISyntaxException use) {
          _app.message(use.toString(), _app.MESSAGE_ERREUR);
        }
      }
      if (elem != _elementSelectionne && elem.estDeplaceable()) {
        setElementSelectionne(elem);
      } else {
        setElementSelectionne(null);
      } 
    }
    if (source instanceof JButton) {
      PanelJeu pj = getPlateau((JButton)source); 
      if (pj != null && pj instanceof PanelTableauJeu) 
        ((TbiRituel)_app).setPanneau("TableauJeu");
      if (pj != null && pj instanceof PanelTableauPresence) 
        ((TbiRituel)_app).setPanneau("TableauPresence");
    }
  }

  @Override  
  public Dimension getDimBoutonTableau() {
    return new Dimension(80, 20);
  }
 
  @Override 
  public JButton getBoutonPlateau(String libelle) {
    BoutonRituel b = new BoutonRituel(libelle);
    b.addActionListener(this);
    return b;
  }
  
  @Override
  public void agenceLesComposants() {
    super.agenceLesComposants();
    // calcul des dimensions 
    Rectangle dim = getBounds();
    _largeurPlateau = dim.width;
    _hauteurPlateau = dim.height;
    double rLH = (double)_eTTresChaud.getLargeur() / (double)_eTTresChaud.getHauteur();
    double pctZonesSaisies = 1.2;
    int nbLignes = 0;
    if (((TbiRituel)_app).isElementMeteoSelected("Température")) nbLignes++;
    if (((TbiRituel)_app).isElementMeteoSelected("Ciel")) nbLignes++;
    if (((TbiRituel)_app).isElementMeteoSelected("Précipitation")) nbLignes++;
    if (((TbiRituel)_app).isElementMeteoSelected("Vent")) nbLignes++;
    int lEtiquette, hEtiquette;
    int lZoneEtiquettes = _largeurPlateau;
    int hZoneEtiquettes = _hauteurPlateau - _dyEtiquette - _eMeteo.getHauteur() - _dyEtiquette;     
    // On se base sur la hauteur des étiquettes
    hEtiquette = (int)((hZoneEtiquettes - (4+(nbLignes-1)+4)*_dyEtiquette) / nbLignes / pctZonesSaisies); // 4 _dyEtiquette de chaque côté pour la marge 4 + 3 + 4
    if (hEtiquette > _eTTresChaud.getHauteur())
      hEtiquette = _eTTresChaud.getHauteur();  
    lEtiquette = (int)(hEtiquette * rLH);
    // répartition horizontale
    int maxColonnes = getMaxColonnes();
    int dxHorizontal = (lZoneEtiquettes - (maxColonnes*lEtiquette) - lEtiquette - ((maxColonnes-1)*_dxEtiquette))/3;
    int margeGauche = dxHorizontal;
    int margeDroite = dxHorizontal; 

    // le titre
    float rMeteo = (float)_eMeteo.getLargeur() / (float)_eMeteo.getHauteur(); 
    int hMeteo = (int)(_hauteurPlateau * ((float)_eMeteo.getHauteur()/768f));
    int lMeteo = (int)(hMeteo * rMeteo);
    int x = (_largeurPlateau-lMeteo)/2;
    int y = _dyEtiquette + dim.y;      
    _eMeteo.setDimension(lMeteo, hMeteo);
    _eMeteo.setIniLocation(x,y);
    
    int dyVertical = (int)((hZoneEtiquettes - (nbLignes*hEtiquette*pctZonesSaisies))/(nbLignes+1));
    int margeHaute = _dyEtiquette*2+_eMeteo.getHauteur()+dyVertical;
    int margeBasse = dyVertical;

    x = margeGauche;
    y = margeHaute;
// les températures    
    int yTemperature = y;
	  if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_TTresChaud)) {
      _eTTresChaud.setIniLocation(x,y);
      _eTTresChaud.setDimension(lEtiquette, hEtiquette);
      x += lEtiquette + _dxEtiquette;
    }
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_TChaud)) {
      _eTChaud.setIniLocation(x,y);
      _eTChaud.setDimension(lEtiquette, hEtiquette);    
      x += lEtiquette + _dxEtiquette;
    }  
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_TTempere)) {      
      _eTTempere.setIniLocation(x,y);
      _eTTempere.setDimension(lEtiquette, hEtiquette);    
      x += lEtiquette + _dxEtiquette;
    }
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_TFroid)) {      
      _eTFroid.setIniLocation(x,y);
      _eTFroid.setDimension(lEtiquette, hEtiquette);    
      x += lEtiquette + _dxEtiquette;
    }
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_TTresFroid)) {      
      _eTTresFroid.setIniLocation(x,y);
      _eTTresFroid.setDimension(lEtiquette, hEtiquette);
    }
    
    // les cieux
    x = margeGauche;
    if (((TbiRituel)_app).isElementMeteoSelected("Température"))
      y += hEtiquette + dyVertical;
    int yCiel = y;
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_COrage)) {    
      _eCOrage.setIniLocation(x,y);
      _eCOrage.setDimension(lEtiquette, hEtiquette);
      x += lEtiquette + _dxEtiquette;
    }
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_CCouvert)) {
      _eCCouvert.setIniLocation(x,y);
      _eCCouvert.setDimension(lEtiquette, hEtiquette);    
      x += lEtiquette + _dxEtiquette;
    }
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_CEnsoleilleBlanc)) {      
      _eCEnsoleilleBlanc.setIniLocation(x,y);
      _eCEnsoleilleBlanc.setDimension(lEtiquette, hEtiquette);    
      x += lEtiquette + _dxEtiquette;
    }
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_CBrouillard)) {    
      _eCBrouillard.setIniLocation(x,y);
      _eCBrouillard.setDimension(lEtiquette, hEtiquette);    
      x += lEtiquette + _dxEtiquette;
    }
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_CEnsoleille)) {
      _eCEnsoleille.setIniLocation(x,y);
      _eCEnsoleille.setDimension(lEtiquette, hEtiquette);
    }

    // les précipitations
    x = margeGauche;
    if (((TbiRituel)_app).isElementMeteoSelected("Ciel"))
      y += hEtiquette + dyVertical;
    int yPrecipitation = y;
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_PSec)) {    
      _ePSec.setIniLocation(x,y);
      _ePSec.setDimension(lEtiquette, hEtiquette);
      x += lEtiquette + _dxEtiquette;
    }
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_PPluie)) {  
      _ePPluie.setIniLocation(x,y);
      _ePPluie.setDimension(lEtiquette, hEtiquette);    
      x += lEtiquette + _dxEtiquette;
    }
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_PAverse)) {    
      _ePAverse.setIniLocation(x,y);
      _ePAverse.setDimension(lEtiquette, hEtiquette);    
      x += lEtiquette + _dxEtiquette;
    }
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_PGrele)) {      
      _ePGrele.setIniLocation(x,y);
      _ePGrele.setDimension(lEtiquette, hEtiquette);    
      x += lEtiquette + _dxEtiquette;
    }
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_PNeige)) {      
      _ePNeige.setIniLocation(x,y);
      _ePNeige.setDimension(lEtiquette, hEtiquette);
    }

    // les vents
    x = margeGauche;
    if (((TbiRituel)_app).isElementMeteoSelected("Précipitation"))
      y += hEtiquette + dyVertical;
    int yVent = y;
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_VNul)) {        
      _eVNul.setIniLocation(x,y);
      _eVNul.setDimension(lEtiquette, hEtiquette);
      x += lEtiquette + _dxEtiquette;
    }
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_VFort)) {
      _eVFort.setIniLocation(x,y);
      _eVFort.setDimension(lEtiquette, hEtiquette);    
      x += lEtiquette + _dxEtiquette;
    }
    if (((TbiRituel)_app).getAvecElementMeteo(TbiRituel._V_VTempete)) {      
      _eVTempete.setIniLocation(x,y);
      _eVTempete.setDimension(lEtiquette, hEtiquette);
    } 

    // les zones de saisies
    if (((TbiRituel)_app).isElementMeteoSelected("Température")) {
      _eSTemperature.setDimension((int)(lEtiquette*pctZonesSaisies), (int)(hEtiquette*pctZonesSaisies));
      _eSTemperature.setIniLocation(_largeurPlateau-margeDroite-(int)(lEtiquette*pctZonesSaisies), yTemperature-((int)(hEtiquette*(pctZonesSaisies-1)/2)));
    }
    if (((TbiRituel)_app).isElementMeteoSelected("Ciel")) {
      _eSCiel.setDimension((int)(lEtiquette*pctZonesSaisies), (int)(hEtiquette*pctZonesSaisies));
      _eSCiel.setIniLocation(_largeurPlateau-margeDroite-(int)(lEtiquette*pctZonesSaisies), yCiel-((int)(hEtiquette*(pctZonesSaisies-1)/2)));
    }
    if (((TbiRituel)_app).isElementMeteoSelected("Précipitation")) {
      _eSPrecipitation.setDimension((int)(lEtiquette*pctZonesSaisies), (int)(hEtiquette*pctZonesSaisies));
      _eSPrecipitation.setIniLocation(_largeurPlateau-margeDroite-(int)(lEtiquette*pctZonesSaisies), yPrecipitation-((int)(hEtiquette*(pctZonesSaisies-1)/2)));
    }
    if (((TbiRituel)_app).isElementMeteoSelected("Vent")) {
      _eSVent.setDimension((int)(lEtiquette*pctZonesSaisies), (int)(hEtiquette*pctZonesSaisies));
      _eSVent.setIniLocation(_largeurPlateau-margeDroite-(int)(lEtiquette*pctZonesSaisies), yVent-((int)(hEtiquette*(pctZonesSaisies-1)/2)));
    }
    placementsComplets();
    repaint();   
  }
  
  public int getMaxColonnes() {
    int max = 0;
    int maxc = 0;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_TTresChaud)) maxc++;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_TChaud)) maxc++;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_TTempere)) maxc++;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_TFroid)) maxc++;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_TTresFroid)) maxc++;
    max = maxc;
    maxc = 0;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_COrage)) maxc++;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_CCouvert)) maxc++;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_CEnsoleille)) maxc++;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_CEnsoleilleBlanc)) maxc++;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_CBrouillard)) maxc++;                    
    if(maxc > max) max = maxc;
    maxc = 0;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_PSec)) maxc++;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_PPluie)) maxc++;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_PAverse)) maxc++;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_PGrele)) maxc++;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_PNeige)) maxc++;
    if(maxc > max) max = maxc;
    maxc = 0;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_VNul)) maxc++;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_VBrise)) maxc++;
    if (((TbiRituel)_app).isElementMeteoSelected(TbiRituel._V_VTempete)) maxc++;
    if(maxc > max) max = maxc;
    return max;  
  }
  
  public void	eventDispatched(AWTEvent e) {
    if (e instanceof MouseEvent) {
      MouseEvent me = (MouseEvent)e;
      Point p = me.getLocationOnScreen();
      _mouseX = p.x;
      _mouseY = p.y;
      super.eventDispatched(e);
      if (isShowing() && _elementSelectionne != null && _rectEcran != null) {
        Point spotMouse = _elementSelectionne.getSpotMouse();
        Point posParent = _elementSelectionne.getParent().getLocationOnScreen();
        _elementSelectionne.setLocation(_mouseX-posParent.x-spotMouse.x, _mouseY-posParent.y-spotMouse.y);        
      }
    } else {
        super.eventDispatched(e);
    }      
  }
  
  public void setElementSelectionne(Element e) {
    if (e == null) { // on relache l'élément
      if (estCeUneTemperature(_elementSelectionne)) 
        analyseBonneZoneSaisie(_eSTemperature, _elementSelectionne);
      if (estCeUnCiel(_elementSelectionne)) 
        analyseBonneZoneSaisie(_eSCiel, _elementSelectionne);
      if (estCeUnePrecipitation(_elementSelectionne)) 
        analyseBonneZoneSaisie(_eSPrecipitation, _elementSelectionne);
      if (estCeUnVent(_elementSelectionne)) 
        analyseBonneZoneSaisie(_eSVent, _elementSelectionne);
      if (_elementSelectionne != null)
        _pMain.setComponentZOrder(_elementSelectionne, _oldZOrder);  
    } else { // on vient de cliquer sur l'élément
        _oldZOrder = _pMain.getComponentZOrder(e);
        _pMain.setComponentZOrder(e, 0);
        if (e == _eSelTemperature)
          _eSelTemperature = null;
        if (e == _eSelCiel)
          _eSelCiel = null;
        if (e == _eSelPrecipitation)
          _eSelPrecipitation = null;
        if (e == _eSelVent)
          _eSelVent = null;
    }
    _elementSelectionne = e;
  }

  public Element getElementSelectionne() {
    return _elementSelectionne;
  }

  public boolean estCeUneTemperature(Element e) {
    return (e == _eTTresChaud ||
            e == _eTChaud ||
            e == _eTTempere ||
            e == _eTFroid ||
            e == _eTTresFroid);                                                            
  }

  public boolean estCeUnCiel(Element e) {
    return (e == _eCOrage ||
            e == _eCCouvert ||
            e == _eCEnsoleilleBlanc ||
            e == _eCBrouillard ||
            e == _eCEnsoleille); 
  }

  public boolean estCeUnePrecipitation(Element e) {
    return (e == _ePSec ||
            e == _ePPluie ||
            e == _ePAverse ||
            e == _ePGrele ||
            e == _ePNeige);                                                            
  }

  public boolean estCeUnVent(Element e) {
    return (e == _eVNul ||
            e == _eVFort ||
            e == _eVTempete); 
  }
  
  public void analyseBonneZoneSaisie(Element zS, Element e) {
    Point pIni = e.getIniLocation();
    if (zS.contient(_mouseX, _mouseY) && 
        ((_eSTemperature == zS && _eSelTemperature == null) ||
         (_eSCiel == zS && _eSelCiel == null) ||
         (_eSPrecipitation == zS && _eSelPrecipitation == null) ||
         (_eSVent == zS && _eSelVent == null))) {
      centreElement(zS, e);
      if (zS == _eSTemperature)
        _eSelTemperature = e;
      if (zS == _eSCiel)
        _eSelCiel = e;
      if (zS == _eSPrecipitation)
        _eSelPrecipitation = e;
      if (zS == _eSVent)
        _eSelVent = e;
    } else {
      e.setLocation(pIni.x, pIni.y);
    }
    placementsComplets();
  }
  
  private void centreElement(Element eZs, Element e) {
    Rectangle rEZs = eZs.getBounds();
    Rectangle rE = e.getBounds();
    e.setLocation(rEZs.x+rEZs.width/2-rE.width/2, rEZs.y+rEZs.height/2-rE.height/2);
  }

  private boolean placementsComplets() {
    if (((_eSTemperature.contient(_eTTresChaud) ||
         _eSTemperature.contient(_eTChaud) ||
         _eSTemperature.contient(_eTTempere) ||
         _eSTemperature.contient(_eTFroid) ||
         _eSTemperature.contient(_eTTresFroid)) || !((TbiRituel)_app).isElementMeteoSelected("Température")) &&
        ((_eSCiel.contient(_eCOrage) ||
         _eSCiel.contient(_eCCouvert) ||
         _eSCiel.contient(_eCEnsoleilleBlanc) ||
         _eSCiel.contient(_eCBrouillard) ||
         _eSCiel.contient(_eCEnsoleille)) || !((TbiRituel)_app).isElementMeteoSelected("Ciel")) &&
        ((_eSPrecipitation.contient(_ePSec) ||
         _eSPrecipitation.contient(_ePPluie) ||
         _eSPrecipitation.contient(_ePAverse) ||
         _eSPrecipitation.contient(_ePGrele) ||
         _eSPrecipitation.contient(_ePNeige)) || !((TbiRituel)_app).isElementMeteoSelected("Précipitation")) &&
        ((_eSVent.contient(_eVNul) ||
         _eSVent.contient(_eVFort) ||
         _eSVent.contient(_eVTempete)) || !((TbiRituel)_app).isElementMeteoSelected("Vent"))) {
      return true;
    }
    return false;
  }
  
  public Element getTemperature() {
    return _eSelTemperature;
  }

  public Element getCiel() {
    return _eSelCiel;
  }
  
  public Element getPrecipitation() {
    return _eSelPrecipitation;
  }
  
  public Element getVent() {
    return _eSelVent;
  }
  
}